#!/bin/sh

#Defines
install=0
verbose=0

dir_list="etc usr/bin usr/doc/pos usr/include/pos usr/lib usr/sbin usr/share/pos var/lib/misc" 
pos_dir=""  #Directories with "pos" in their names. 
font_dir="usr/share/pos/fonts"
kernel_version=`uname -r`
cmd_dir="src/cmd"
bin_dir="usr/bin"

mkdir_p()
{
   pdir=""
   dir_parts=`echo $1|sed 's/\// /g'`
   for path_part in `echo $dir_parts`; do
      pdir="$pdir/$path_part"
      if [ ! -d $pdir ]; then
         mkdir $pdir
      fi
   done
}         

remove_dir()
{
   if [ $# ]; then
      subdirectory=$1
   fi 

   if [ -d $subdirectory ]; then
      for file in $subdirectory/*; do
         if [ -d $file ]; then 
            remove_dir $file
         else
            if [ $verbose -eq 1 ];then
               echo "Removing /$file"
            fi
            rm -f /$file
         fi
      done
   fi
}


help_opt()
{
   echo -e "The proper usage is as follows :
           ./install_pos.sh [-i,--install -u,--uninstall, -h,--help]\n\
         	-i,--install    Installs the POS subsystem
                -u,--uninstall  Uninstalls the POS subsystem
                -v,--verbose    Print messages about the install or uninstall
                -h,--help       print this help message
         "
   exit 1
}

 
parse_opts()
{
   #The option parsing is portable to systems without getopt installed.

   if which getopt >/dev/null; then
      INSTALL_OPTS=`getopt -o iuvh -l install,uninstall,verbose,help  -- "$@"` 

      eval set -- "$INSTALL_OPTS"
   fi

   while [ $# -gt 0 ] ; do
      case "$1" in
         -i|--install)   if [ $install -ne 0 ]; then 
                            echo "You cannot install and uninstall!"
                            help_opt
                         fi
                         install=1; shift ;;
         -u|--uninstall) if [ $install -ne 0 ]; then 
                            echo "You cannot install and uninstall!"
                            help_opt
                         fi
                         install=2; shift ;; 
         -v|--verbose)   verbose=1; shift;;
         -h|--help)      help_opt ; shift 2 ;;
         --) shift ; break ;;
         *) help_opt ; exit 1 ;;
      esac
   done                 
}

create_file_list()
{
   subdirectory=$1
 
   if [ -d $subdirectory ]; then
      for file in $subdirectory/*; do
         if [ -d $file ]; then
            create_file_list $file
         else
            echo "/$file" >> /usr/share/pos/.pos_files
            if [ $verbose -eq 1 ];then
               echo -e "/$file"
            fi
         fi
      done
   fi              
}

prepare_uninstall()
{
   if [ ! -d /usr/share/pos ]; then
      mkdir_p "/usr/share/pos"
   fi

   if [ $verbose -eq 1 ];then
      echo -e "Creating an uninstallation file list in /usr/share/pos/.pos_files\nPackage files:\n\n"
   fi

   for directory in *; do   
      if [ ! -f $directory ]; then
         create_file_list $directory
      fi       
   done
}

warn_about_previous_install()
{
   echo -e "A previous installation was detected.\nWe recommend that you uninstall before installing."
   echo -n "Would you like to install anyway?"

   while true; do
      read continuing
      case "$continuing" in
         [yY]|yes|YES|Yes)  return 0; break ;;
         [nN]|no|NO|No)     return 1; break ;;
         *)                 echo -n "Please enter \"yes\" or \"no\":"  ;;
      esac
   done 
}

 
parse_opts $@

#The distribution tarball should contain only these directories, but list them to be certain.
if [ "$install" -eq 1 ]; then

   #Do a perfunctory check and warn for previous installations.

   if [ -f /usr/share/pos/.pos_files ]; then
      if ! warn_about_previous_install; then
         exit 0
      fi
   fi

   #Generate a list of files to uninstall and an uninstallation prefix.

   if [ $verbose -eq 1 ];then
      echo -e "Installing the IBM Point of Sale Subsystem\n\n"
   fi

   prepare_uninstall
   for directory in $dir_list; do
      if [ ! -d $directory ]; then
         echo -e "Directory $directory does not exist!\
                  This POS distribution is incomplete./n\Aborting installation..."
         exit 1;
      fi

      if [ ! -d /$directory ]; then
         mkdir_p $directory
      fi
      
      for files in $directory; do

         if [ $verbose -eq 1 ];then
            echo -e "Installing the files in $files"
         fi

         cp -pR $files/* /$directory >/dev/null 2>&1
      done
   done


   #Driver Installation
 
   #DCS Installation

   #Install the init script

   if [ ! -d /etc/rc.d/init.d ]; then
      mkdir_p /etc/rc.d/init.d
   fi      

   cp -f etc/rc.d/init.d/aipstart /etc/rc.d/init.d/

   if [ $verbose -eq 1 ];then
      echo -e "\nEstablishing symbolic links for IBM POS libraries\n"
      ldconfig -v 2>&1|egrep "libaip"
   else
      ldconfig
   fi

   #Add init script links
  
   if [ $verbose -eq 1 ];then
      echo "Creating runlevel symbolic links for init script"
   fi

   /sbin/chkconfig --add aipstart
 
   if [ $? != 0 ]; then
      echo "Could not add init script"
      exit 1;
   fi

   #Add symlink for developers
   if [ ! -h /usr/lib/libaiplib.so ]; then
      ln -s `ls /usr/lib/libaiplib.*.*.*` /usr/lib/libaiplib.so
   fi

   if [ $verbose -eq 1 ];then
      echo "Creating device nodes /dev/aipikbps, /dev/aipdcs, and /dev/aiptchmouse" 
   fi

   #Make device files.

   if [ ! -c /dev/aipikbps ]; then
      /bin/mknod /dev/aipikbps c 246 0
   fi   

   if [ ! -c /dev/aipdcs ]; then
      /bin/mknod /dev/aipdcs c 245 0
   fi

   if [ ! -c /dev/aiptchmouse ]; then
      /bin/mknod /dev/aiptchmouse c 248 0
   fi
   
   if [ $verbose -eq 1 ];then
      echo "Creating device nodes /dev/input/touch, /dev/input/touch0, and /dev/touch1" 
   fi

   if [ ! -c /dev/input/touch0 ]; then
      /bin/mknod /dev/input/touch0 c 13 248
   fi
   
   if [ ! -c /dev/input/touch1 ]; then
      /bin/mknod /dev/input/touch1 c 13 249
   fi
   
   if [ ! -c /dev/input/touch ]; then
      /bin/mknod /dev/input/touch c 13 250
   fi      

elif [ "$install" -eq 2 ]; then
   
   if [ $verbose -eq 1 ];then
      echo -e "Uninstalling the IBM Point of Sale Subsystem\n\n"
   fi

   #Shut down the IBM POS Subsystem
   /etc/rc.d/init.d/aipstart stop
   
   # Remove the keyboard driver
   if [ "`grep aipikbps /proc/modules`" ]; then   
       if [ $verbose -eq 1 ];then
          echo "Removing POS keyboard driver" 
       fi

       for I in 0 1 2 3 4 5 6 7 8 9; do
           rmmod aipikbps >/dev/null 2>&1
           if [ "`grep aipikbps /proc/modules`" ]; then
               usleep 200000
	   else
	       break
	   fi
       done
   fi

   #Remove the runlevel symbolic links

   if [ $verbose -eq 1 ];then
      echo "Removing runlevel symbolic links for init script"
   fi

   if [ -a /etc/rc.d/init.d/aipstart ]; then
      /sbin/chkconfig --del aipstart
   fi

   for directory in $dir_list; do
      pos_dir=`expr $directory : '\(.*pos\).*'`
      if [ "$pos_dir" != "" ]; then
         if [ -d /$pos_dir ]; then
            if [ $verbose -eq 1 ];then
               echo "Removing /$pos_dir"
            fi
            rm -Rf /$pos_dir
         fi
      fi
   done

   curr_dir=`pwd`
   #Changed removal to remove based on tarball contents
   if expr "$curr_dir" : ".*ibmposs-linux-[0-9]*\.[0-9]*\.[0-9]*.*" >/dev/null; then  
      for directory in *; do
         if [ ! -f $directory ]; then
            remove_dir $directory
         fi
      done

   else   
   #Rely upon our file "database".
      if [ -a /usr/share/pos/.pos_files ]; then
         exec 3< /usr/share/pos/.pos_files
         while read line 0<&3; do
            if [ $verbose -eq 1 ];then
               echo "Removing $line"
            fi
            rm -f $line
         done  
         exec 3<&-
      else
         echo "Aborting installation:  Could not determine which files to remove."
      fi
   fi

   #Let's remove ldconfig symbolic links now

   for file in /usr/lib/libaip*; do
      if [ -h $file ]; then
         if [ $verbose -eq 1 ];then
            echo "Removing $file"
         fi
         rm -f $file
      fi
   done

   if [ $verbose -eq 1 ];then
      echo "Removing the device nodes /dev/aipikbps, /dev/aipdcs, and /dev/aiptchmouse" 
   fi

   if [ -a /dev/aipikbps ]; then
      rm -f /dev/aipikbps 
   fi   

   if [ -a /dev/aipdcs ]; then
      rm -f /dev/aipdcs 
   fi   

   if [ -a /dev/aiptchmouse ]; then
      rm -f /dev/aiptchmouse 
   fi   

   if [ $verbose -eq 1 ];then
      echo "Removing the device nodes /dev/input/touch, /dev/input/touch0, and /dev/input/touch1" 
   fi

   if [ -a /dev/input/touch0 ]; then
      rm -f /dev/input/touch0 
   fi
   
   if [ -a /dev/input/touch1 ]; then
      rm -f /dev/input/touch1 
   fi
   
   if [ -a /dev/input/touch ]; then
      rm -f /dev/input/touch 
   fi
   
   #Clean up /var/log
   if [ $verbose -eq 1 ];then
      echo "Removing log files from /var/log" 
   fi

   if [  -f /var/log/aipctrl.log ]; then
      rm -f /var/log/aipctrl.log
   fi          

   for num in 1 2 3; do   
      if [  -f /var/log/aipctrl.log.$num ]; then
         rm -f /var/log/aipctrl.log.$num
      fi          
   done
   
   if [  -f /var/log/aipanpos.log ]; then
      rm -f /var/log/aipanpos.log
   fi 
    
   if [  -f /var/log/aipdebug.log ]; then
      rm -f /var/log/aipdebug.log
   fi 
    
   if [  -f /var/log/aiptrace.log ]; then
      rm -f /var/log/aiptrace.log
   fi  
else
   help_opt 
fi
